@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 # Copyright 2018 New Vector Ltd
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -34,7 +33,11 @@ class WellKnownBuilder:
         self._config = hs.config
 
     def get_well_known(self):
-        result = {"m.homeserver": {"base_url": self._config.public_baseurl}}
+        # if we don't have a public_baseurl, we can't help much here.
+        if self._config.server.public_baseurl is None:
+            return None
+
+        result = {"m.homeserver": {"base_url": self._config.server.public_baseurl}}
 
         if self._config.default_identity_server:
             result["m.identity_server"] = {
